/* 
 * Copyright (C) 2000-1 Aaron Lav, asl2@pobox.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <rfftw.h>
#define __USE_XOPEN
// for M_SQRT_2
#define __ISO_C9X
// for exp2
#include <math.h>

#include <assert.h>
#include <inttypes.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <vector>
#include <map>
#include <algorithm>



#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "soundcard.h"
#include "stuff.h"
#include "sq.h"

Samples::Chunk::Chunk ()
{
}

fftw_real Samples::Chunk::calc_one_diff (Chunk *c)
{
  // we could rewrite this to be loudness[i]^2 + c->loudness[i]^2
  // - 2 * loudness[i] * c->loudness[i], precomputing l^2 and c->l^2,
  // but that just replaces a subtraction and a mult. with 
  // an addition, a subtraction, and a mult (and a mult by 2).
  // and won't have any better cache behavior.

  fftw_real t = 0;
  assert (loudness.size () == c->loudness.size ());
  for (unsigned int i = 0; i < loudness.size (); i++) {
    t += sq(loudness[i] - c->loudness[i]);
  }
  assert (!isnan (t));
  return t;
}

bool Samples::Chunk::diff_t::operator< (const struct diff_t& a) const
{
  return dist < a.dist;
  // does this meet stability reqts
  // in the face of aggressive inlining and maybe-conforming-maybe-not FP?
};

void Samples::Chunk::calc_diffs (Chunk *chs, int n, int start)
{
  // XXX rewrite this to be more efficient!
  vector<diff_t>  diffs_tmp (n);
  diff_tot = 0;
  for (int i = 0; i < n; i++) {
    int which = (i + start) % n;
    diff_t d = {which, calc_one_diff (chs + which)};
    diffs_tmp[i] = d;
    diff_tot += d.dist;
  }
  // important to use stable_sort.  In the presence of silence,
  // we end up w/ lots of bits which are the same.  Naive code
  // will quickly drop into a stable loop (with period much less
  // than that of the entire track, that is...).
  stable_sort (diffs_tmp.begin(),diffs_tmp.end ());
  diffs_tmp.resize (255); // make 255 configable XXX
 
  diffs = diffs_tmp;
}

int Samples::Chunk::get_similar (float prob)
{
#if 1
  for (unsigned int i = 0; i < diffs.size () - 1; i++) {
    float r = (1.0 * rand ()) / RAND_MAX;
    if (r < prob) {
      return diffs[i].which;
    }
  }

#else
  float pw = 0.25;
  float rand_val = (rand () * pow(diffs[diffs.size() - 1].dist *
		    (1 - prob),pw)) / RAND_MAX;
  float running = 0;
  for (int i = 0; i < diffs.size () - 1; i++) {
    running += diffs [i + 1].dist;
    if (rand_val < pow(running,pw)) {
      return diffs[i].which;
    }
  }
#endif
  return diffs [diffs.size () -1].which;
}


void Samples::Chunk::diffs_to_pixels (DiffStore *ds)
{
  ds->set_len (diffs.size ());
  for (unsigned int j = 0; j < diffs.size (); j++) {
    if (!ds->set_diff (diffs[j].which, diffs[j].dist, j))
      return;
  }
}

 


