/* 
 * Copyright (C) 2000-1 Aaron Lav, asl2@pobox.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <inttypes.h>
#define _GNU_SOURCE
#include <getopt.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <vector>
#include <map>
#include <algorithm>


#include <rfftw.h>

#include <audiofile.h>

#include "soundcard.h"
#include "stuff.h"

#include "gtkplayer.h"
#include "sq.h"



struct option longopts [] = {
  {"read_mat", required_argument,  NULL, 'r'},
  {"write_mat", required_argument, NULL, 'w'},
  {"write_loudness", required_argument, NULL, 'l'},
  {"write_distance", required_argument, NULL, 'd'},
  {"chunk_duration", required_argument, NULL, 's'},
  {"echo_len", required_argument, NULL, 'e'},
  {"echo_amp", required_argument, NULL, 'a'},
  {0,0,0,0}
};
int main (int argc, char **argv)
{
  const char optstring [] = "r:w:l:d:s:e:a:";
  float dur = 0.1;
  int echo_len = 3;
  float echo_amp = 0.3;
  printf ("Dissociated Studio, by Aaron Lav, is licensed under the GPL\n");
  printf ("See http://www.pobox.com/~asl2/music/dissoc_studio/ for details\n");

  char *fn;
  const char *to_write= NULL, *to_read = NULL, *loud_write = NULL,
    *diff_write = NULL;
  int op;
  while (EOF != (op = getopt_long (argc, argv, optstring,longopts,
			      NULL))) {
    switch (op) {
    case 'e':
      echo_len = atoi (optarg);
      break;
    case 'a':
      echo_amp = atof (optarg);
      break;
    case 'r':
      to_read = optarg;
      break;
    case 'w':
      to_write = optarg;
      break;
    case 'l':
      loud_write = optarg;
      break;
    case 'd':
      diff_write = optarg;
      break;
    case 's': {
      float f = atof (optarg);
      if (f) 
	dur = f;
      else
	printf ("Bad duration %s", optarg);
      break;
    }
    default:
      printf ("Bad arg %c\n", op);
      break;
    }
  }
  if (argc == optind) {
    printf ("Must specify file name\n");
    exit (255);
  }
  else
    fn = argv [optind];
  Samples s;
  s.set_chunk_dur (dur); // in seconds
  s.read (fn); 
  SoundPlayer *sp = new SoundPlayer;
  sp->request_samprate (s.get_rate ());
  sp->request_nchan (1);
  assert (sp->create ("/dev/dsp"));

  

  if (to_read) {
#if 0 
    if (!read_chunks (s.cs, to_read, s.chunk_cnt)) // XXX fix or lose read_chunks
      exit (255);
#endif
    printf ("Read %s\n", to_read);
  } else {
    s.set_echo_len (echo_len);
    s.set_echo_amp (echo_amp);
    s.make_chunks ();
  }

  bool gui = true;
  if (gui) {
    GUIPlayer p(&argc,&argv, &s, sp);
    Vis::Parm vp = {s.get_chunk_cnt(), &s};
    for (int i = 0; 1;i++) {
      Vis *v = mk_vis (i,  &p);
      if (!v)
	break;

      v->set_parms (&vp);
      p.add_vis (v);
    }
    p.run ();
  } else {
    Player p (&s, sp);
  }
}












