/* 
 * Copyright (C) 2000-1 Aaron Lav, asl2@pobox.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <inttypes.h>
#include <audiofile.h>
#include <rfftw.h>


int main (int argc, char **argv)
{
  if (argc != 4) {
    printf ("Usage: %s start_freq end_freq filename\n", argv[0]);
    return 255;
  }

  int start_freq = atoi (argv [1]), end_freq = atoi (argv [2]);
  const char *fn = argv [3];
  AFfilesetup fs = afNewFileSetup ();
  int samples_per_sec = 44100;
  afInitRate (fs, AF_DEFAULT_TRACK, samples_per_sec);
  afInitFileFormat (fs, AF_FILE_WAVE);
  afInitSampleFormat (fs, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
  afInitByteOrder (fs, AF_DEFAULT_TRACK, AF_BYTEORDER_LITTLEENDIAN);
  afInitChannels (fs, AF_DEFAULT_TRACK, 1);

  AFfilehandle afh = afOpenFile (fn, "w", fs);

  int buf_sz = 44100;
  fftw_plan plan = rfftw_create_plan (buf_sz, FFTW_COMPLEX_TO_REAL,
				 FFTW_MEASURE | FFTW_IN_PLACE);
  fftw_plan fw_plan = rfftw_create_plan (buf_sz, FFTW_REAL_TO_COMPLEX,
				 FFTW_MEASURE | FFTW_IN_PLACE);

  for (int i = start_freq; i < end_freq; i++) {
    int16_t s [buf_sz];
    fftw_real d [buf_sz];
    memset (d, 0, sizeof d);
    // frequencies have spacing sample_rate / (sample_size * 2)
    d [i * buf_sz * 2 /samples_per_sec] = 1;
    rfftw_one (plan, d, NULL);
    for (int j = 0; j < buf_sz; j++) {
      s[j] = (int16_t) (d[j] * 16384); // 
      if (i == start_freq) {
	printf ("j %d f %f s %d\n", j, d[j], s[j]);
	d[j] = (fftw_real) s[j];
      }
    }
    rfftw_one (fw_plan, d, NULL);
    if (i == start_freq)
      for (int j = 0; j < buf_sz; j++) {
	printf ("%d %f\n", j, d[j]);
      }
    afWriteFrames (afh, AF_DEFAULT_TRACK, (void *) s, buf_sz);
  }
  afCloseFile (afh);
}
