/* 
 * Copyright (C) 2000-1 Aaron Lav, asl2@pobox.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <inttypes.h>
#define _GNU_SOURCE
#include <getopt.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <vector>
#include <map>
#include <algorithm>


#include <rfftw.h>

#include <audiofile.h>

#include "soundcard.h"
#include "stuff.h"

#include "player.h"

Player::Player (Samples *s, SoundPlayer *sp) : 
  samp(s), sound (sp)
{
  clear_hist ();
  max_chunks_buf_in_OSS = (sound->get_samples_writeable() + 
		       samp->get_chunk_sz () - 1) / samp->get_chunk_sz();
  transition_alg = 0;
}

void Player::clear_hist ()
{
  play_times.resize (samp->get_chunk_cnt());
  prev_chunk.resize (samp->get_chunk_cnt(), -1);

  cur_playing = 0;
  n_played = 0;
  max_age = 1;
}

float Player::get_jump_probability ()
{
  return 0.1;
}

Player::~Player ()
{
}

void Player::run ()
{
  while (1) {
    printf ("%d ", cur_playing);
    calc_next ();

    time_to_chunk.erase (time_to_chunk.begin());
    // don't need this facility for simple player here, but
    // makes more sense to implement map updating in base calc_next

    sound->write (samp->get_chunk_dat (cur_playing), samp->get_chunk_sz());

    if ((n_played) % 10 == 0) {
      printf ("\n");
    }
  }
}

const int Player::transition_alg_count = 2;
char *Player::transition_alg_names [Player::transition_alg_count] = {
  "Get Similar",
  "Random"};

void Player::calc_next () 
{
  unsigned int skip = 1;
  unsigned int base = (cur_playing + skip) % samp->get_chunk_cnt();
  unsigned int next;

  switch (transition_alg) {
  case 0:
    next = samp-> get_similar (get_jump_probability (), base);
    break;
  case 1: {
    float r = (1.0 * rand ()) / RAND_MAX;
    if (r > get_jump_probability ())
      next = (rand () / 17) % samp->get_chunk_cnt();
    else
      next = base;
    break;
  }
  default:
    printf ("Bad alg %d\n", transition_alg);
    assert (0);
  }

  prev_chunk[next] = cur_playing; 

  cur_playing = next;
  time_to_chunk[n_played] = cur_playing;

  // and trim too-old stuff
  ttc_map::iterator it = time_to_chunk.begin ();
  while (it != time_to_chunk.end () && (n_played - it->first) > max_age) {
    //    printf ("Erasing %d %d at %d\n",  it->first, it->second, n_played);
    play_times[it->second] = it->first;
    time_to_chunk.erase (it);
    it = time_to_chunk.begin ();
  }
  n_played++;
}
