#include <map>
#include <vector>
#include <fftw.h>
#include <rfftw.h>

#include "soundcard.h"
#include "diffstore.h"

class Samples {
public:
  Samples ();

  void set_chunk_dur (float secs);
  float get_chunk_dur ();

  int get_rate (void) {return rate;};
  unsigned int get_echo_len ();
  float get_echo_amp ();
  void set_echo_len (unsigned int len);
  void set_echo_amp (float f);
  size_t get_chunk_sz () {return chunk_sz;};
  size_t get_chunk_cnt () {return chunk_cnt;};


  sample const *get_chunk_dat (size_t chunk_ind);
  // return chunk_sz for short bit at end?  Or truncate?

  void read (const char *fn);
  void make_chunks ();
  void calc_window (); // XXX allow for different types
  void apply_window (fftw_real *buf);
  void calc_sz ();
  unsigned int get_similar (float prob, unsigned int base);
  void copy_and_echo (fftw_real * buf, int c);
  void diffs_to_pixels (DiffStore *ds, unsigned int i);
private:
  void calc_loudness (int c, vector<fftw_real>& calcd_loudness);
  class Chunk {
  public:
    Chunk ();

    void calc_diffs (Chunk *chs, int n, int start);
    int get_similar (float prob);
    void diffs_to_pixels (DiffStore *ds);
    // XXX get_loudness is icky, but only accessible to Samples...
    vector<fftw_real>& get_loudness () {return loudness;};
  private:

    vector<fftw_real> loudness;

    // log (re ^ 2 + im ^ 2), starting w/ 1st non-DC component
    // (psychoacoustically, DC isn't sensed), reduced 
    // (see chunk.cc for description

    // currently, loudness.size () must be the same
    // across all chunks, but this would be easy to fix
    // (only assumed in calc_one_diff)


    struct diff_t {
      int which;
      float dist;
      bool operator< (const struct diff_t& a) const;
    };

    vector<struct diff_t> diffs;
    fftw_real calc_one_diff (Chunk *c);
    double diff_tot;
  };

  rfftw_plan plan;
  Chunk *cs;

  size_t  chunk_sz;
  size_t  chunk_cnt;

  fftw_real *window;
  size_t  d_len;
  int rate;
  float chunk_duration;
  sample *data;
  unsigned int echo_len;
  float echo_amp;
};









